==> 11.5.zip: tasks/app/src/googleplay/java/org/tasks/location/LocationServiceGooglePlay.kt <==
package org.tasks.location

import android.annotation.SuppressLint
import android.app.PendingIntent
import android.content.Context
import android.content.Intent
import com.google.android.gms.location.Geofence
import com.google.android.gms.location.GeofencingRequest
import com.google.android.gms.location.LocationServices
import dagger.hilt.android.qualifiers.ApplicationContext
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import org.tasks.data.MergedGeofence
import org.tasks.data.Place
import javax.inject.Inject
import kotlin.coroutines.suspendCoroutine

class LocationServiceGooglePlay @Inject constructor(
        @ApplicationContext private val context: Context
) : LocationService {
    @SuppressLint("MissingPermission")
    override suspend fun currentLocation(): MapPosition = withContext(Dispatchers.IO) {
        suspendCoroutine { cont ->
            LocationServices
                    .getFusedLocationProviderClient(context)
                    .lastLocation
                    .addOnSuccessListener {
                        cont.resumeWith(Result.success(MapPosition(it.latitude, it.longitude)))
                    }
                    .addOnFailureListener { cont.resumeWith(Result.failure(it)) }
        }
    }

    @SuppressLint("MissingPermission")
    override fun addGeofences(geofence: MergedGeofence) {
        LocationServices
                .getGeofencingClient(context)
                .addGeofences(
                GeofencingRequest.Builder().addGeofence(toGoogleGeofence(geofence)).build(),
                PendingIntent.getBroadcast(
                        context,
                        0,
                        Intent(context, GoogleGeofenceTransitionIntentService.Broadcast::class.java),
                        PendingIntent.FLAG_UPDATE_CURRENT))
    }

    override fun removeGeofences(place: Place) {
        LocationServices
                .getGeofencingClient(context)
                .removeGeofences(listOf(place.id.toString()))
    }

    private fun toGoogleGeofence(geofence: MergedGeofence): Geofence {
        var transitionTypes = 0
        if (geofence.arrival) {
            transitionTypes = transitionTypes or GeofencingRequest.INITIAL_TRIGGER_ENTER
        }
        if (geofence.departure) {
            transitionTypes = transitionTypes or GeofencingRequest.INITIAL_TRIGGER_EXIT
        }
        return Geofence.Builder()
                .setCircularRegion(geofence.latitude, geofence.longitude, geofence.radius.toFloat())
                .setRequestId(geofence.uid)
                .setTransitionTypes(transitionTypes)
                .setExpirationDuration(Geofence.NEVER_EXPIRE)
                .build()
    }
}

==> 11.10.1.zip: tasks/app/src/googleplay/java/org/tasks/location/LocationServiceGooglePlay.kt <==
package org.tasks.location

import android.annotation.SuppressLint
import android.app.PendingIntent
import android.content.Context
import android.content.Intent
import com.google.android.gms.location.Geofence
import com.google.android.gms.location.GeofencingRequest
import com.google.android.gms.location.LocationServices
import dagger.hilt.android.qualifiers.ApplicationContext
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import org.tasks.data.MergedGeofence
import org.tasks.data.Place
import javax.inject.Inject
import kotlin.coroutines.suspendCoroutine

class LocationServiceGooglePlay @Inject constructor(
        @ApplicationContext private val context: Context
) : LocationService {
    @SuppressLint("MissingPermission")
    override suspend fun currentLocation(): MapPosition = withContext(Dispatchers.IO) {
        suspendCoroutine { cont ->
            LocationServices
                    .getFusedLocationProviderClient(context)
                    .lastLocation
                    .addOnSuccessListener {
                        cont.resumeWith(
                                it?.let { Result.success(MapPosition(it.latitude, it.longitude)) }
                                        ?: Result.failure(NullPointerException())
                        )
                    }
                    .addOnFailureListener { cont.resumeWith(Result.failure(it)) }
        }
    }

    @SuppressLint("MissingPermission")
    override fun addGeofences(geofence: MergedGeofence) {
        LocationServices
            .getGeofencingClient(context)
            .addGeofences(
                GeofencingRequest.Builder().addGeofence(toGoogleGeofence(geofence)).build(),
                PendingIntent.getBroadcast(
                    context,
                    0,
                    Intent(context, GoogleGeofenceTransitionIntentService.Broadcast::class.java),
                    PendingIntent.FLAG_IMMUTABLE or PendingIntent.FLAG_UPDATE_CURRENT
                )
            )
    }

    override fun removeGeofences(place: Place) {
        LocationServices
                .getGeofencingClient(context)
                .removeGeofences(listOf(place.id.toString()))
    }

    private fun toGoogleGeofence(geofence: MergedGeofence): Geofence {
        var transitionTypes = 0
        if (geofence.arrival) {
            transitionTypes = transitionTypes or GeofencingRequest.INITIAL_TRIGGER_ENTER
        }
        if (geofence.departure) {
            transitionTypes = transitionTypes or GeofencingRequest.INITIAL_TRIGGER_EXIT
        }
        return Geofence.Builder()
                .setCircularRegion(geofence.latitude, geofence.longitude, geofence.radius.toFloat())
                .setRequestId(geofence.uid)
                .setTransitionTypes(transitionTypes)
                .setExpirationDuration(Geofence.NEVER_EXPIRE)
                .build()
    }
}
